@@ -136,7 +136,7 @@ DLL_EXPORT int parse_args (char* p, int maxargc, char** pargv, int* pargc)
         {
             char delim = *p;
             if (p == *pargv) *pargv = p+1;
-            while (*++p && *p != delim); if (!*p) break; // find end of quoted string
+            while (*++p && *p != delim) {}; if (!*p) break; // find end of quoted string
         }
 
         *p++ = 0; // mark end of arg
