@@ -1,30 +1,14 @@
-/* CCKDDIAG.C   (c) Copyright Roger Bowler, 1999-2007                */
-/*       CCKD diagnostic tool                                        */
+/* CCKDDIAG.C   (c) Copyright Greg Smith, 2000-2009                  */
+/*              (c) Copyright James M. Morrison, 2003                */
+/*              CCKD diagnostic tool                                 */
+
 /* 2003-02-07 James M. Morrison initial implementation               */
 /* portions borrowed from cckdcdsk & other CCKD code                 */
 
-// $Id: cckddiag.c,v 1.28 2008/11/04 04:50:45 fish Exp $
-
 /*-------------------------------------------------------------------*/
 /* Diagnostic tool to display various CCKD data                      */
 /*-------------------------------------------------------------------*/
 
-// $Log: cckddiag.c,v $
-// Revision 1.28  2008/11/04 04:50:45  fish
-// Ensure consistent utility startup
-//
-// Revision 1.27  2008/06/22 05:54:30  fish
-// Fix print-formatting issue (mostly in tape modules)
-// that can sometimes, in certain circumstances,
-// cause herc to crash.  (%8.8lx --> I32_FMTX, etc)
-//
-// Revision 1.26  2007/06/23 00:04:03  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.25  2006/12/08 09:43:17  jj
-// Add CVS message log
-//
-
 #include "hstdinc.h"
 
 /* TODO: add FBA support or write cfbadiag                           */
@@ -102,7 +86,7 @@ int    x;
 /*-------------------------------------------------------------------*/
 /* clean - cleanup before exit                                       */
 /*-------------------------------------------------------------------*/
-void clean() {
+void clean(void) {
     close(fd);
     free(l1);                          /* L1TAB buffer               */
     free(l2);                          /* L2TAB buffer               */
@@ -151,7 +135,7 @@ int readpos(
     if (debug) 
         fprintf(stdout, 
                 "READPOS reading buf addr "PTR_FMTx" length %"SIZE_T_FMT"d (0x"SIZE_T_FMTX")\n",
-                buf, len, len);
+                (uintptr_t)buf, len, len);
     if (read(fd, buf, len) < (ssize_t)len) {
         fprintf(stdout, _("cckddiag: read error: %s\n"),
                         strerror(errno));
@@ -491,7 +475,7 @@ char            pathname[MAX_PATH];     /* file path in host format  */
 
     /* open the file */
     hostpath(pathname, fn, sizeof(pathname));
-    fd = open(pathname, O_RDONLY | O_BINARY);
+    fd = hopen(pathname, O_RDONLY | O_BINARY);
     if (fd < 0) {
         fprintf(stdout,
                 _("cckddiag: error opening file %s: %s\n"),
@@ -520,7 +504,7 @@ char            pathname[MAX_PATH];     /* file path in host format  */
             fprintf(stdout, 
                     "DASD table entry not found for devtype 0x%2.2X\n",
                     devhdr.devtype);
-            clean(fd);
+            clean();
             exit(5);
         }
     }
@@ -534,7 +518,7 @@ char            pathname[MAX_PATH];     /* file path in host format  */
                     "DASD table entry not found for "
                     "devtype 0x%2.2X\n",
                     DEFAULT_FBA_TYPE);
-            clean(fd);
+            clean();
             exit(6);
         }
     }
