@@ -1,79 +1,6 @@
-// ====================================================================
-//           Hercules LAN Channel Station Support
-// ====================================================================
-//
-// Copyright (C) 2002-2009 by James A. Pierson    (original author)
-// Copyright (C) 2002-2009 by David B. Trout      (current maintainer)
-//
-// $Id: ctc_lcs.c 5125 2009-01-23 12:01:44Z bernard $
-//
-// $Log$
-// Revision 1.81  2008/12/23 14:01:22  rbowler
-// Allow symbol substitution in OAT files
-//
-// Revision 1.80  2008/12/22 14:14:03  rbowler
-// Correct typographical error in message HHCLC032E
-//
-// Revision 1.79  2008/11/04 05:56:31  fish
-// Put ensure consistent create_thread ATTR usage change back in
-//
-// Revision 1.78  2008/11/03 15:31:57  rbowler
-// Back out consistent create_thread ATTR modification
-//
-// Revision 1.77  2008/10/18 09:32:21  fish
-// Ensure consistent create_thread ATTR usage
-//
-// Revision 1.76  2008/08/19 21:36:37  fish
-// Init LCS interface ASAP to fix wrong MAC being used
-//
-// Revision 1.75  2008/08/15 04:40:49  fish
-// Trace LCS Command Frame packets as they are received,
-// ensure iMaxFrameBufferSize greater than minimum allowed.
-//
-// Revision 1.74  2008/07/17 07:42:15  fish
-// (extremely minor comment change only)
-//
-// Revision 1.73  2008/07/17 07:19:12  fish
-// Fix FCS (Frame Check Sequence) bug in LCS_Write function
-// and other minor bugs.
-//
-// Revision 1.71  2008/07/17 03:30:40  fish
-// CTC/LCS cosmetic-only changes -- part 1
-// (no actual functionality was changed!)
-//
-// Revision 1.70  2008/05/22 18:27:25  fish
-// Fix read timeout bug in LCS logic
-//
-// Revision 1.69  2008/02/07 00:29:04  rbowler
-// Solaris build support by Jeff Savit
-//
-// Revision 1.68  2008/01/11 21:33:21  fish
-// new 'ctc' command to enable/disable debug option on demand
-//
-// Revision 1.67  2007/11/21 22:54:14  fish
-// Use new BEGIN_DEVICE_CLASS_QUERY macro
-//
-// Revision 1.66  2007/08/17 00:59:40  fish
-// Backout 28 Jul fix for LCS (fix was only meant for CTCI)
-//
-// Revision 1.65  2007/07/29 02:02:44  fish
-// Fix day-1 CTCI/LCS bug found by Vince Weaver [vince@deater.net]
-//
-// Revision 1.64  2007/06/23 00:04:05  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.63  2007/02/25 15:05:46  fish
-// Fix crash in LCS close if devinit of incomplete group
-//
-// Revision 1.62  2007/01/14 22:17:35  rbowler
-// Correct compile error introduced by rev 1.61
-//
-// Revision 1.61  2007/01/14 08:03:30  fish
-// correct minor (benign(?)) bug in LCS_QueryIPAssists, fix HHCLC011I and related deug msgs in LCS_PortThread to display IP address in correct byte-order.
-//
-// Revision 1.60  2006/12/08 09:43:19  jj
-// Add CVS message log
-//
+/* CTC_LCS.C    (c) Copyright James A. Pierson, 2002-2012            */
+/*              (c) Copyright "Fish" (David B. Trout), 2002-2011     */
+/*              Hercules LAN Channel Station Support                 */
 
 #include "hstdinc.h"
 
@@ -180,14 +107,38 @@ static void*    LCS_PortThread( PLCSPORT pLCSPORT );
 static int      LCS_EnqueueEthFrame( PLCSDEV pLCSDEV, BYTE   bPort,
                                      BYTE*   pData,   size_t iSize );
 
-static void*    LCS_InitReplyFrame( PLCSDEV pLCSDEV,
-                                    size_t  iSize, PLCSCMDHDR pCmdFrame );
+static int      LCS_EnqueueReplyFrame( PLCSDEV pLCSDEV, PLCSCMDHDR pReply,
+                                                        size_t     iSize );
 
 static int      BuildOAT( char* pszOATName, PLCSBLK pLCSBLK );
 static char*    ReadOAT( char* pszOATName, FILE* fp, char* pszBuff );
 static int      ParseArgs( DEVBLK* pDEVBLK, PLCSBLK pLCSBLK,
                            int argc, char** argv );
 
+// ====================================================================
+//                       Helper macros
+// ====================================================================
+
+#define INIT_REPLY_FRAME( reply, pCmdFrame )                \
+                                                            \
+    memset( &(reply), 0, sizeof( reply ));                  \
+    memcpy( &(reply), (pCmdFrame), sizeof( LCSCMDHDR ));    \
+    STORE_HW( (reply).bLCSCmdHdr.hwReturnCode, 0x0000 )
+
+#define ENQUEUE_REPLY_FRAME( pLCSDEV, reply )                               \
+                                                                            \
+    while                                                                   \
+    (1                                                                      \
+        && LCS_EnqueueReplyFrame( (pLCSDEV), (PLCSCMDHDR) &(reply),         \
+                                                    sizeof( reply )) != 0   \
+        &&  (pLCSDEV)->pLCSBLK->Port[(pLCSDEV)->bPort].fd != -1             \
+        && !(pLCSDEV)->pLCSBLK->Port[(pLCSDEV)->bPort].fCloseInProgress     \
+    )                                                                       \
+    {                                                                       \
+        TRACE("** ENQUEUE_REPLY_FRAME() failed...\n");                      \
+        SLEEP( 1 );                                                         \
+    }
+
 // ====================================================================
 //                    find_group_device
 // ====================================================================
@@ -1233,29 +1184,18 @@ void  LCS_SDC( DEVBLK* pDEVBLK,   BYTE   bOpCode,
 
 static void  LCS_Startup( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 {
-    PLCSSTDFRM  pStdCmdReplyFrame;
-    PLCSSTRTFRM pStartupCmdFrame;
+    LCSSTRTFRM  reply;
     PLCSPORT    pLCSPORT;
+    U16         iOrigMaxFrameBufferSize;
 
-    pLCSPORT = &pLCSDEV->pLCSBLK->Port[pLCSDEV->bPort];
+    INIT_REPLY_FRAME( reply, pCmdFrame );
 
-    // Get a pointer to the next available reply frame
-    pStdCmdReplyFrame = (PLCSSTDFRM)LCS_InitReplyFrame( pLCSDEV,
-                                              sizeof( LCSSTDFRM ),
-                                              pCmdFrame );
-
-    // NOTE: pStdCmdReplyFrame now points to a reply frame
-    // the first part of which is an LCSCMDHDR which, except for
-    // the 'hwOffset' field, is an exact copy of the LCSCMDHDR
-    // that was passed to us...
-
-    pStdCmdReplyFrame->bLCSCmdHdr.bLanType      = LCS_FRMTYP_ENET;
-    pStdCmdReplyFrame->bLCSCmdHdr.bRelAdapterNo = pLCSDEV->bPort;
-
-    pStartupCmdFrame = (PLCSSTRTFRM)pCmdFrame;
+    reply.bLCSCmdHdr.bLanType      = LCS_FRMTYP_ENET;
+    reply.bLCSCmdHdr.bRelAdapterNo = pLCSDEV->bPort;
 
     // Save the max buffer size parameter
-    FETCH_HW( pLCSDEV->iMaxFrameBufferSize, pStartupCmdFrame->hwBufferSize );
+    iOrigMaxFrameBufferSize = pLCSDEV->iMaxFrameBufferSize;
+    FETCH_HW( pLCSDEV->iMaxFrameBufferSize, ((PLCSSTRTFRM)pCmdFrame)->hwBufferSize );
 
     // Make sure it doesn't exceed our compiled maximum
     if (pLCSDEV->iMaxFrameBufferSize > sizeof(pLCSDEV->bFrameBuffer))
@@ -1265,7 +1205,7 @@ static void  LCS_Startup( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
                   pLCSDEV->pDEVBLK[1]->devnum,
                   pLCSDEV->iMaxFrameBufferSize,
                   sizeof( pLCSDEV->bFrameBuffer ) );
-        pLCSDEV->iMaxFrameBufferSize = sizeof(pLCSDEV->bFrameBuffer);
+        pLCSDEV->iMaxFrameBufferSize = iOrigMaxFrameBufferSize;
     }
 
     // Make sure it's not smaller than the compiled minimum size
@@ -1276,9 +1216,11 @@ static void  LCS_Startup( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
                   pLCSDEV->pDEVBLK[1]->devnum,
                   pLCSDEV->iMaxFrameBufferSize,
                   CTC_MIN_FRAME_BUFFER_SIZE );
-        pLCSDEV->iMaxFrameBufferSize = sizeof(pLCSDEV->bFrameBuffer);
+        pLCSDEV->iMaxFrameBufferSize = iOrigMaxFrameBufferSize;
     }
 
+    pLCSPORT = &pLCSDEV->pLCSBLK->Port[pLCSDEV->bPort];
+
     VERIFY( TUNTAP_SetIPAddr( pLCSPORT->szNetDevName, "0.0.0.0" ) == 0 );
     VERIFY( TUNTAP_SetMTU   ( pLCSPORT->szNetDevName,  "1500"   ) == 0 );
 
@@ -1290,7 +1232,7 @@ static void  LCS_Startup( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
     }
 #endif // OPTION_TUNTAP_SETMACADDR
 
-    STORE_HW( pStdCmdReplyFrame->bLCSCmdHdr.hwReturnCode, 0x0000 );
+    ENQUEUE_REPLY_FRAME( pLCSDEV, reply );
 
     pLCSDEV->fStarted = 1;
 }
@@ -1301,22 +1243,14 @@ static void  LCS_Startup( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 
 static void  LCS_Shutdown( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 {
-    PLCSSTDFRM  pStdCmdReplyFrame;
+    LCSSTDFRM   reply;
 
-    // Get a pointer to the next available reply frame
-    pStdCmdReplyFrame = (PLCSSTDFRM)LCS_InitReplyFrame( pLCSDEV,
-                                              sizeof( LCSSTDFRM ),
-                                              pCmdFrame );
+    INIT_REPLY_FRAME( reply, pCmdFrame );
 
-    // NOTE: pStdCmdReplyFrame now points to a reply frame
-    // the first part of which is an LCSCMDHDR which, except for
-    // the 'hwOffset' field, is an exact copy of the LCSCMDHDR
-    // that was passed to us...
+    reply.bLCSCmdHdr.bLanType      = LCS_FRMTYP_ENET;
+    reply.bLCSCmdHdr.bRelAdapterNo = pLCSDEV->bPort;
 
-    pStdCmdReplyFrame->bLCSCmdHdr.bLanType      = LCS_FRMTYP_ENET;
-    pStdCmdReplyFrame->bLCSCmdHdr.bRelAdapterNo = pLCSDEV->bPort;
-
-    STORE_HW( pStdCmdReplyFrame->bLCSCmdHdr.hwReturnCode, 0x0000 );
+    ENQUEUE_REPLY_FRAME( pLCSDEV, reply );
 
     pLCSDEV->fStarted = 0;
 }
@@ -1327,13 +1261,15 @@ static void  LCS_Shutdown( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 
 static void  LCS_StartLan( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 {
+    LCSSTDFRM   reply;
     PLCSPORT    pLCSPORT;
 #ifdef OPTION_TUNTAP_DELADD_ROUTES
     PLCSRTE     pLCSRTE;
 #endif // OPTION_TUNTAP_DELADD_ROUTES
-    PLCSSTDFRM  pStdCmdReplyFrame;
     int         nIFFlags;
 
+    INIT_REPLY_FRAME( reply, pCmdFrame );
+
     pLCSPORT = &pLCSDEV->pLCSBLK->Port[pLCSDEV->bPort];
 
     // Serialize access to eliminate ioctl errors
@@ -1400,17 +1336,7 @@ static void  LCS_StartLan( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
     }
 #endif // OPTION_TUNTAP_DELADD_ROUTES
 
-    // Get a pointer to the next available reply frame
-    pStdCmdReplyFrame = (PLCSSTDFRM)LCS_InitReplyFrame( pLCSDEV,
-                                              sizeof( LCSSTDFRM ),
-                                              pCmdFrame );
-
-    // NOTE: pStdCmdReplyFrame now points to a reply frame
-    // the first part of which is an LCSCMDHDR which, except for
-    // the 'hwOffset' field, is an exact copy of the LCSCMDHDR
-    // that was passed to us...
-
-    STORE_HW( pStdCmdReplyFrame->bLCSCmdHdr.hwReturnCode, 0 );
+    ENQUEUE_REPLY_FRAME( pLCSDEV, reply );
 }
 
 // ====================================================================
@@ -1419,12 +1345,14 @@ static void  LCS_StartLan( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 
 static void  LCS_StopLan( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 {
+    LCSSTDFRM   reply;
     PLCSPORT    pLCSPORT;
-    PLCSSTDFRM  pStdCmdReplyFrame;
 #ifdef OPTION_TUNTAP_DELADD_ROUTES
     PLCSRTE     pLCSRTE;
 #endif // OPTION_TUNTAP_DELADD_ROUTES
 
+    INIT_REPLY_FRAME( reply, pCmdFrame );
+
     pLCSPORT = &pLCSDEV->pLCSBLK->Port[pLCSDEV->bPort];
 
     // Serialize access to eliminate ioctl errors
@@ -1474,17 +1402,7 @@ static void  LCS_StopLan( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
     // FIXME: Really need to iterate through the devices and close
     //        the TAP interface if all devices have been stopped.
 
-    // Get a pointer to the next available reply frame
-    pStdCmdReplyFrame = (PLCSSTDFRM)LCS_InitReplyFrame( pLCSDEV,
-                                              sizeof( LCSSTDFRM ),
-                                              pCmdFrame );
-
-    // NOTE: pStdCmdReplyFrame now points to a reply frame
-    // the first part of which is an LCSCMDHDR which, except for
-    // the 'hwOffset' field, is an exact copy of the LCSCMDHDR
-    // that was passed to us...
-
-    STORE_HW( pStdCmdReplyFrame->bLCSCmdHdr.hwReturnCode, 0 );
+    ENQUEUE_REPLY_FRAME( pLCSDEV, reply );
 }
 
 // ====================================================================
@@ -1493,21 +1411,13 @@ static void  LCS_StopLan( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 
 static void  LCS_QueryIPAssists( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 {
+    LCSQIPFRM   reply;
     PLCSPORT    pLCSPORT;
-    PLCSQIPFRM  pStdCmdReplyFrame;
+
+    INIT_REPLY_FRAME( reply, pCmdFrame );
 
     pLCSPORT = &pLCSDEV->pLCSBLK->Port[pLCSDEV->bPort];
 
-    // Get a pointer to the next available reply frame
-    pStdCmdReplyFrame = (PLCSQIPFRM)LCS_InitReplyFrame( pLCSDEV,
-                                              sizeof( LCSQIPFRM ),
-                                              pCmdFrame );
-
-    // NOTE: pStdCmdReplyFrame now points to a reply frame
-    // the first part of which is an LCSCMDHDR which, except for
-    // the 'hwOffset' field, is an exact copy of the LCSCMDHDR
-    // that was passed to us...
-
 #if defined( WIN32 )
 
     // FIXME: TunTap32 *does* support TCP/IP checksum offloading
@@ -1568,10 +1478,12 @@ static void  LCS_QueryIPAssists( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 
 #endif // WIN32
 
-    STORE_HW( pStdCmdReplyFrame->hwNumIPPairs,         0x0000 );
-    STORE_HW( pStdCmdReplyFrame->hwIPAssistsSupported, pLCSPORT->sIPAssistsSupported );
-    STORE_HW( pStdCmdReplyFrame->hwIPAssistsEnabled,   pLCSPORT->sIPAssistsEnabled   );
-    STORE_HW( pStdCmdReplyFrame->hwIPVersion,          0x0004 );
+    STORE_HW( reply.hwNumIPPairs,         0x0000 );
+    STORE_HW( reply.hwIPAssistsSupported, pLCSPORT->sIPAssistsSupported );
+    STORE_HW( reply.hwIPAssistsEnabled,   pLCSPORT->sIPAssistsEnabled   );
+    STORE_HW( reply.hwIPVersion,          0x0004 );
+
+    ENQUEUE_REPLY_FRAME( pLCSDEV, reply );
 }
 
 // ====================================================================
@@ -1580,27 +1492,17 @@ static void  LCS_QueryIPAssists( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 
 static void  LCS_LanStats( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 {
+    LCSLSTFRM    reply;
     PLCSPORT     pLCSPORT;
-    PLCSLSTFRM   pStdCmdReplyFrame;
     int          fd;
     struct ifreq ifr;
     BYTE*        pPortMAC;
     BYTE*        pIFaceMAC;
 
+    INIT_REPLY_FRAME( reply, pCmdFrame );
+
     pLCSPORT = &pLCSDEV->pLCSBLK->Port[pLCSDEV->bPort];
 
-    // Get a pointer to the next available reply frame
-    pStdCmdReplyFrame = (PLCSLSTFRM)LCS_InitReplyFrame( pLCSDEV,
-                                              sizeof( LCSLSTFRM ),
-                                              pCmdFrame );
-
-    // NOTE: pStdCmdReplyFrame now points to a reply frame
-    // the first part of which is an LCSCMDHDR which, except for
-    // the 'hwOffset' field, is an exact copy of the LCSCMDHDR
-    // that was passed to us...
-
-    STORE_HW( pStdCmdReplyFrame->bLCSCmdHdr.hwReturnCode, 0x0000 );
-
     fd = socket( AF_INET, SOCK_STREAM, IPPROTO_IP );
 
     if( fd == -1 )
@@ -1608,7 +1510,7 @@ static void  LCS_LanStats( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
         logmsg( _("HHCLC007E Error in call to socket: %s.\n"),
                 strerror( HSO_errno ) );
         // FIXME: we should probably be returning a non-zero hwReturnCode
-        // STORE_HW( pStdCmdReplyFrame->bLCSCmdHdr.hwReturnCode, 0x0001 );
+        // STORE_HW( reply.bLCSCmdHdr.hwReturnCode, 0x0001 );
         return;
     }
 
@@ -1626,7 +1528,7 @@ static void  LCS_LanStats( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
         logmsg( _("HHCLC008E ioctl error on device %s: %s.\n"),
                 pLCSPORT->szNetDevName, strerror( errno ) );
         // FIXME: we should probably be returning a non-zero hwReturnCode
-        // STORE_HW( pStdCmdReplyFrame->bLCSCmdHdr.hwReturnCode, 0x0002 );
+        // STORE_HW( reply.bLCSCmdHdr.hwReturnCode, 0x0002 );
         return;
     }
     pIFaceMAC  = (BYTE*) ifr.ifr_hwaddr.sa_data;
@@ -1653,14 +1555,25 @@ static void  LCS_LanStats( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 
         memcpy( pPortMAC, pIFaceMAC, IFHWADDRLEN );
 
-        snprintf(pLCSPORT->szMACAddress, sizeof(pLCSPORT->szMACAddress),
+        snprintf(pLCSPORT->szMACAddress, sizeof(pLCSPORT->szMACAddress)-1,
             "%2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X", *(pPortMAC+0), *(pPortMAC+1),
             *(pPortMAC+2), *(pPortMAC+3), *(pPortMAC+4), *(pPortMAC+5));
     }
 
-    memcpy( pStdCmdReplyFrame->MAC_Address, pIFaceMAC, IFHWADDRLEN );
+    memcpy( reply.MAC_Address, pIFaceMAC, IFHWADDRLEN );
+
+    /* Respond with a different MAC address for the LCS side */
+    /* unless the TAP mechanism is designed as such          */
+    /* cf : hostopts.h for an explanation                    */
+#if !defined(OPTION_TUNTAP_LCS_SAME_ADDR)
+
+    reply.MAC_Address[5]++;
+
+#endif
 
     // FIXME: Really should read /proc/net/dev to retrieve actual stats
+
+    ENQUEUE_REPLY_FRAME( pLCSDEV, reply );
 }
 
 // ====================================================================
@@ -1669,22 +1582,14 @@ static void  LCS_LanStats( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 
 static void  LCS_DefaultCmdProc( PLCSDEV pLCSDEV, PLCSCMDHDR pCmdFrame )
 {
-    PLCSSTDFRM  pStdCmdReplyFrame;
+    LCSSTDFRM   reply;
 
-    // Get a pointer to the next available reply frame
-    pStdCmdReplyFrame = (PLCSSTDFRM)LCS_InitReplyFrame( pLCSDEV,
-                                              sizeof( LCSSTDFRM ),
-                                              pCmdFrame );
+    INIT_REPLY_FRAME( reply, pCmdFrame );
 
-    // NOTE: pStdCmdReplyFrame now points to a reply frame
-    // the first part of which is an LCSCMDHDR which, except for
-    // the 'hwOffset' field, is an exact copy of the LCSCMDHDR
-    // that was passed to us...
+    reply.bLCSCmdHdr.bLanType      = LCS_FRMTYP_ENET;
+    reply.bLCSCmdHdr.bRelAdapterNo = pLCSDEV->bPort;
 
-    pStdCmdReplyFrame->bLCSCmdHdr.bLanType      = LCS_FRMTYP_ENET;
-    pStdCmdReplyFrame->bLCSCmdHdr.bRelAdapterNo = pLCSDEV->bPort;
-
-    STORE_HW( pStdCmdReplyFrame->bLCSCmdHdr.hwReturnCode, 0x0000 );
+    ENQUEUE_REPLY_FRAME( pLCSDEV, reply );
 }
 
 // ====================================================================
@@ -1751,8 +1656,7 @@ static void*  LCS_PortThread( PLCSPORT pLCSPORT )
                 break;
             logmsg( _("HHCLC042E Port %2.2X: Read error: %s\n"),
                 pLCSPORT->bPort, strerror( errno ) );
-            SLEEP(1);           // (purposeful long delay)
-            continue;
+            break;
         }
 
         if( pLCSPORT->pLCSBLK->fDebug )
@@ -2038,7 +1942,7 @@ static int  LCS_EnqueueEthFrame( PLCSDEV pLCSDEV, BYTE   bPort,
                                  pLCSDEV->iFrameOffset );
 
     // Increment offset to NEXT available slot (after ours)
-    pLCSDEV->iFrameOffset += sizeof(LCSETHFRM) + iSize;
+    pLCSDEV->iFrameOffset += (U16)(sizeof(LCSETHFRM) + iSize);
 
     // Plug updated offset to next frame into our frame header
     STORE_HW( pLCSEthFrame->bLCSHdr.hwOffset, pLCSDEV->iFrameOffset );
@@ -2064,39 +1968,43 @@ static int  LCS_EnqueueEthFrame( PLCSDEV pLCSDEV, BYTE   bPort,
 }
 
 // ====================================================================
-//                       LCS_InitReplyFrame
+//                       LCS_EnqueueReplyFrame
 // ====================================================================
 //
-// Returns a pointer to the next available frame slot of iSize bytes
-// of type LCSCMDHDR (i.e. the partially pre-initialized reply frame)
-//
-// As a part of frame setup, initializes the reply frame with basic
-// information that is provided in the original frame for which this
-// is a reply frame for. Note that only Command frames have replies.
+// Copy a pre-built LCS Command Frame reply frame of iSize bytes
+// to the next available frame slot. Returns 0 on success, -1 and
+// errno set to ENOBUFS on failure (no room (yet) in o/p buffer).
+// The LCS device lock must NOT be held when called.
 //
 // --------------------------------------------------------------------
 
-static void*  LCS_InitReplyFrame( PLCSDEV pLCSDEV,
-                                  size_t  iSize, PLCSCMDHDR pCmdFrame )
+static int  LCS_EnqueueReplyFrame( PLCSDEV pLCSDEV, PLCSCMDHDR pReply,
+                                                    size_t     iSize )
 {
     PLCSCMDHDR  pReplyCmdFrame;
 
     obtain_lock( &pLCSDEV->Lock );
 
+    // Ensure we dont overflow the buffer
+    if( ( pLCSDEV->iFrameOffset +           // Current buffer Offset
+          iSize +                           // Size of reply frame
+          sizeof(pReply->bLCSHdr.hwOffset)) // Size of Frame terminator
+        > pLCSDEV->iMaxFrameBufferSize )    // Size of Frame buffer
+    {
+        release_lock( &pLCSDEV->Lock );
+        errno = ENOBUFS;                    // No buffer space available
+        return -1;                          // (-1==failure)
+    }
+
     // Point to next available LCS Frame slot in our buffer...
     pReplyCmdFrame = (PLCSCMDHDR)( pLCSDEV->bFrameBuffer +
                                    pLCSDEV->iFrameOffset );
 
+    // Copy the reply frame into the frame buffer slot...
+    memcpy( pReplyCmdFrame, pReply, iSize );
+
     // Increment buffer offset to NEXT next-available-slot...
-    pLCSDEV->iFrameOffset += iSize;
-
-    // Initialize our Reply LCS Command Frame...
-    memset( pReplyCmdFrame, 0, iSize );
-
-    // Use the LCS Command Frame header that was passed to us
-    // as a template for the Reply LCS Command Frame we're to
-    // build...
-    memcpy( pReplyCmdFrame, pCmdFrame, sizeof( LCSCMDHDR ) );
+    pLCSDEV->iFrameOffset += (U16) iSize;
 
     // Store offset of next frame
     STORE_HW( pReplyCmdFrame->bLCSHdr.hwOffset, pLCSDEV->iFrameOffset );
@@ -2106,7 +2014,7 @@ static void*  LCS_InitReplyFrame( PLCSDEV pLCSDEV,
 
     release_lock( &pLCSDEV->Lock );
 
-    return pReplyCmdFrame;
+    return 0;   // success
 }
 
 // ====================================================================
@@ -2354,6 +2262,7 @@ static int  BuildOAT( char* pszOATName, PLCSBLK pLCSBLK )
         // Read next record from the OAT file
         if( !ReadOAT( pszOATName, fp, szBuff ) )
         {
+            fclose( fp );
             return 0;
         }
 
@@ -2366,7 +2275,7 @@ static int  BuildOAT( char* pszOATName, PLCSBLK pLCSBLK )
 #if defined(OPTION_CONFIG_SYMBOLS)
         // Make a copy of the OAT statement with symbols resolved
         pszStatement = resolve_symbol_string( szBuff );
-#else 
+#else
         // Make a copy of the OAT statement
         pszStatement = strdup( szBuff );
 #endif
@@ -2768,6 +2677,7 @@ HDL_DEVICE_SECTION;
     HDL_DEVICE(3088, ctcadpt_device_hndinfo );
     HDL_DEVICE(CTCI, ctci_device_hndinfo    );
     HDL_DEVICE(CTCT, ctct_device_hndinfo    );
+    HDL_DEVICE(CTCE, ctce_device_hndinfo    );
     HDL_DEVICE(VMNET,vmnet_device_hndinfo   );
 #if defined(WIN32)
     HDL_DEVICE(CTCI-W32,ctci_device_hndinfo );
