@@ -1,8 +1,6 @@
 /* DASDTAB.C    (c) Copyright Roger Bowler, 1999-2009                */
 /*              Hercules Supported DASD definitions                  */
 
-// $Id: dasdtab.c 5444 2009-08-09 16:36:42Z hsg001 $
-
 /*-------------------------------------------------------------------*/
 /* This module contains the tables that define the attributes of     */
 /* each DASD device and control unit supported by Hercules.          */
@@ -14,38 +12,6 @@
 /* (used with permission)                                            */
 /*-------------------------------------------------------------------*/
 
-// $Log$
-// Revision 1.40  2008/11/02 21:45:07  rbowler
-// Correction to dasdtab entries for 3375 and 3380
-//
-// Revision 1.39  2008/10/27 15:10:32  rbowler
-// Revert to 3990 as default control unit for 3390 devices
-//
-// Revision 1.38  2008/02/13 00:30:22  gsmith
-// Fix 3880 SNSID length (thanks De!)
-//
-// Revision 1.37  2007/06/23 00:04:08  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.36  2007/06/20 04:09:54  gsmith
-// make 3990-6 default control unit for 3390 devices
-//
-// Revision 1.35  2007/06/07 19:15:05  kleonard
-// Document circumvention for DSF X'0A' command reject
-//
-// Revision 1.34  2007/05/04 19:28:38  kleonard
-// Circumvent command reject for DSF X'0A' command
-//
-// Revision 1.33  2007/03/06 22:54:19  gsmith
-// Fix ckd RDC response
-//
-// Revision 1.32  2007/02/15 00:10:04  gsmith
-// Fix ckd RCD, SNSS, SNSID responses
-//
-// Revision 1.31  2006/12/08 09:43:20  jj
-// Add CVS message log
-//
-
 #include "hstdinc.h"
 
 #define _DASDTAB_C_
@@ -89,7 +55,7 @@ static CKDDEV ckdtab[] = {
  {"3380",      0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
  {"3380-1",    0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
  {"3380-A",    0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
- {"3380-B",    0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"}, 
+ {"3380-B",    0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
  {"3380-D",    0x3380,0x06,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
  {"3380-J",    0x3380,0x16,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
  {"3380-2",    0x3380,0x0a,0x20,0x0e, 1770,2,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
@@ -120,18 +86,70 @@ static CKDDEV ckdtab[] = {
 /*-------------------------------------------------------------------*/
 /* CKD control unit definitions                                      */
 /*-------------------------------------------------------------------*/
+/*                                                                   */
+/*                                                                   */
+/* 3880, 3990, 2105 and higher coding                                */
+/* ----------------------------------                                */
+/*                                                                   */
+/* Model coding bits (RDC byte 2):                                   */
+/*                                                                   */
+/*   11.. .... 3880 Speed Matching Buffer (LR support)               */
+/*   ..1. .... Non-Gap-Synchronous mode                              */
+/*   .... 1... Cached                                                */
+/*   .... .011 3880-3                                                */
+/*   .... .101 3880-3     Feature 3005 - 3380 AJ4/AK4 Attachment     */
+/*   .... .010 3990-1/2   Nocache/Synchronous                        */
+/*   .... .100 3990-3/6/7 Basic Operation Mode                       */
+/*   .... .001 3990-6/7   Enhanced Mode                              */
+/*                                                                   */
+/*                                                                   */
+/* Features (RDC bytes 6-9):                                         */
+/*                                                                   */
+/*   Byte 6:                                                         */
+/*   1... .... Multiple Burst ECC (ignored, 3380 devices)            */
+/*   .1.. .... Locate Record, Read Tracks                            */
+/*   ..1. .... Reserved                                              */
+/*   ...1 .... Locate Record, Read                                   */
+/*   .... ...1 Reserved for VM minidisk use                          */
+/*             (testing this bit for zero is not reliable as an      */
+/*             indicator that this is not a minidisk; can fail       */
+/*             when minidisk is a fullpack minidisk)                 */
+/*                                                                   */
+/*   Byte 7 - Levels defining byte 8 format                          */
+/*   0000 0000 Not supported by Hercules, must be zero               */
+/*                                                                   */
+/*   Byte 8 - Added feature support                                  */
+/*   000. .... Not supported by Hercules, must be zero               */
+/*   ...1 .... Data striping (required for VSAM Extended Format)     */
+/*   .... 0000 Not supported by Hercules, must be zero               */
+/*                                                                   */
+/*   Byte 9 - Subsystem Program Visible Facilities                   */
+/*   1... .... Cache Fast Write supported (not supported, yet!)      */
+/*             (non-retentive data)                                  */
+/*   .1.. .... Lock Facility supported    (not supported, yet!)      */
+/*   ..1. .... Record Cache supported                                */
+/*   ...1 .... Track Cache supported                                 */
+/*   .... 1... Dual Copy supported        (not supported)            */
+/*   .... .1.. DASD Fast Write supported  (not supported)            */
+/*   .... ..1. Reset Allegiance           (not supported, yet!)      */
+/*   .... ...1 24 Byte Compatibility Sense Format                    */
+/*             (set by code when 3380 on 3390 controller)            */
+/*                                                                   */
+/*                                                                   */
+/*-------------------------------------------------------------------*/
 static CKDCU ckdcutab[] = {
-/*                              func/ type                           */
-/* name          type model code feat code features   ciws --------- */
- {"2314",       0x2314,0x00,0x00,0x00,0x00,0x50000103,0,0,0,0,0,0,0,0},
- {"2835",       0x2835,0x00,0x00,0x00,0x00,0x50000103,0,0,0,0,0,0,0,0},
- {"2841",       0x2841,0x00,0x00,0x00,0x00,0x50000103,0,0,0,0,0,0,0,0},
- {"3830",       0x3830,0x02,0x00,0x00,0x00,0x50000103,0,0,0,0,0,0,0,0},
- {"3880",       0x3880,0x05,0x09,0x00,0x00,0x80000000,0,0,0,0,0,0,0,0},
- {"3990",       0x3990,0xc2,0x10,0x00,0x00,0xd0000002,0x40fa0100,0,0,0,0,0,0,0},
- {"3990-3",     0x3990,0xec,0x06,0x00,0x00,0xd000009e,0x40fa0100,0x41270004,0x423e0040,0,0,0,0,0},
- {"3990-6",     0x3990,0xe9,0x15,0x48,0x15,0x50003097,0x40fa0100,0x41270004,0x423e0060,0,0,0,0,0},
- {"9343",       0x9343,0xe0,0x11,0x00,0x00,0x80000000,0,0,0,0,0,0,0,0}
+/*                              func/ type                                        */
+/* name          type model code feat code features   ciws ---------  senselength */
+ {"2314",       0x2314,0x00,0x00,0x00,0x00,0x00000000,0,0,0,0,0,0,0,0,6},
+ {"2835",       0x2835,0x00,0x00,0x00,0x00,0x00000000,0,0,0,0,0,0,0,0,6},
+ {"2841",       0x2841,0x00,0x00,0x00,0x00,0x00000000,0,0,0,0,0,0,0,0,6},
+ {"3830",       0x3830,0x02,0x00,0x00,0x00,0x00000000,0,0,0,0,0,0,0,0,24},
+ {"3880",       0x3880,0x05,0x09,0x00,0x00,0x80000000,0,0,0,0,0,0,0,0,24},
+ {"3990",       0x3990,0xc2,0x10,0x00,0x00,0xd0000000,0x40fa0100,0,0,0,0,0,0,0,32},
+ {"3990-3",     0x3990,0xec,0x06,0x00,0x00,0xd0001010,0x40fa0100,0x41270004,0x423e0040,0,0,0,0,0,32},
+ {"3990-6",     0x3990,0xe9,0x15,0x48,0x15,0x50001010,0x40fa0100,0x41270004,0x423e0060,0,0,0,0,0,32},
+ {"9343",       0x9343,0xe0,0x11,0x00,0x00,0x80000000,0,0,0,0,0,0,0,0,32}
+/*"2105",       0x2105,0xe8,0x15,0x48,0x15,0x50000037,0x40fa0100,0x41270004,0x423e01a0,0x433e0008,0,0,0,0,32}, */
 } ;
 #define CKDCU_NUM (sizeof(ckdcutab)/CKDCU_SIZE)
 
@@ -182,10 +200,11 @@ static FBADEV fbatab[] = {
  {"9336-25",    0x9336,0x21,0x11,0x10,111,777,512,1672881,0x6310},
  {"9336-x",     0x9336,0x21,0x11,0x10,111,777,512,      0,0x6310},
 
- {"0671-08",    0x0671,0x21,0x12,0x08, 63,630,512, 513072,0x6310},
- {"0671",       0x0671,0x21,0x12,0x00, 63,630,512, 574560,0x6310},
- {"0671-04",    0x0671,0x21,0x12,0x04, 63,630,512, 624456,0x6310},
- {"0671-x",     0x0671,0x21,0x12,0x04, 63,630,512,      0,0x6310}
+/* name          devt class type mdl  bpg bpp size   blks   cu     */
+ {"0671-08",    0x0671,0x21,0x12,0x08, 63,504,512, 513072,0x6310},
+ {"0671",       0x0671,0x21,0x12,0x00, 63,504,512, 574560,0x6310},
+ {"0671-04",    0x0671,0x21,0x12,0x04, 63,504,512, 624456,0x6310},
+ {"0671-x",     0x0671,0x21,0x12,0x04, 63,504,512,      0,0x6310}
 } ;
 #define FBADEV_NUM (sizeof(fbatab)/FBADEV_SIZE)
 
@@ -335,7 +354,10 @@ int altcyls;                            /* Number alternate cyls     */
     devchar[2]  = cu->model;                    // CU model
     store_hw(devchar+3, ckd->devt);             // Device type
     devchar[5]  = ckd->model;                   // Device model
-    store_fw(devchar+6, cu->sctlfeat);          // Device and SD facilities
+    store_fw(devchar+6, cu->sctlfeat |          // Device and SD facilities
+      (cu->devt == 0x3990 &&                    // ... or in 24-byte sense
+       ckd->devt == 0x3380));                   // ... compatability for 3380
+                                                // ... hosted on 3990 controller
     devchar[10] = ckd->class;                   // Device class code
     devchar[11] = ckd->code;                    // Device type code
     store_hw(devchar+12, cyls - altcyls);       // Primary cylinders
@@ -372,7 +394,17 @@ int altcyls;                            /* Number alternate cyls     */
     devchar[47] = 0x01;                         // Track set
     devchar[48] = (BYTE)(ckd->f6);              // F6
     store_hw(devchar+49, ckd->rpscalc);         // RPS factor
-    devchar[51] = MODEL6(cu) ? 0x0f : 0x00;     // reserved byte 51
+    /* Bytes 51-53 are required for VSAM Extended Format */
+    if (MODEL3(cu)) {                           // 3990-3
+        devchar[51] = 0x0f;                     // Feature buffer lower limit
+        devchar[52] = 0x00;                     // Feature buffer upper limit
+        devchar[53] = 0x3f;                     // Feature buffer upper limit
+    }
+    if (MODEL6(cu)) {                           // 3990-6
+        devchar[51] = 0x0f;                     // Feature buffer lower limit
+        devchar[52] = 0x00;                     // Feature buffer upper limit
+        devchar[53] = 0x7f;                     // Feature buffer upper limit
+    }
     devchar[54] = cu->funcfeat;                 // device/CU functions/features
     devchar[56] = cu->typecode;                 // Real CU type code
 
@@ -413,7 +445,8 @@ BYTE buf[256];
                         dev->ckdtab->devt, dev->ckdtab->model);
     for (i = 4; i < 30; i++)
         buf[i] = host_to_guest(buf[i]);
-    store_hw(buf + 30, 0x0300);
+    buf[30] = 0x00;
+    buf[31] = (dev->devnum & 0xFF);
 
     /* Bytes 32-63: NED 2  Node element descriptor for the string */
     store_fw (buf + 32, 0xc4000000);
@@ -421,7 +454,7 @@ BYTE buf[256];
                         dev->ckdtab->devt, dev->ckdtab->model);
     for (i = 36; i < 62; i++)
         buf[i] = host_to_guest(buf[i]);
-    store_hw (buf + 62, 0x0300);
+    store_hw (buf + 62, 0x0000);
 
     /* Bytes 64-95: NED 3  Node element descriptor for the storage director */
     store_fw (buf + 64, 0xd4020000);
@@ -429,7 +462,8 @@ BYTE buf[256];
                         dev->ckdcu->devt, dev->ckdcu->model);
     for (i = 68; i < 94; i++)
         buf[i] = host_to_guest(buf[i]);
-    store_hw (buf + 94, 0x0300);
+    buf[94] = 0x00;
+    buf[95] = (dev->devnum >> 8) & 0xFF;
 
     /* Bytes 96-127: NED 4  Node element descriptor for the subsystem */
     store_fw (buf + 96, 0xF0000001);
@@ -437,7 +471,7 @@ BYTE buf[256];
                         dev->ckdcu->devt);
     for (i = 100; i < 126; i++)
         buf[i] = host_to_guest(buf[i]);
-    store_hw (buf + 126, 0x0300);
+    store_hw (buf + 126, 0x0000);
 
     /* Bytes 128-223: zeroes */
 
