@@ -1,19 +1,6 @@
-/* DEVTYPE.H    (c) Copyright Jan Jaeger, 1999-2007                  */
+/* DEVTYPE.H    (c) Copyright Jan Jaeger, 1999-2009                  */
 /*              Hercules Device Definitions                          */
 
-// $Id: devtype.h,v 1.21 2007/11/21 22:48:45 fish Exp $
-//
-// $Log: devtype.h,v $
-// Revision 1.21  2007/11/21 22:48:45  fish
-// New BEGIN_DEVICE_CLASS_QUERY macro
-//
-// Revision 1.20  2007/06/23 00:04:08  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.19  2006/12/08 09:43:20  jj
-// Add CVS message log
-//
-
 #if !defined(_DEVICES_H)
 
 #define _DEVICES_H
@@ -80,6 +67,7 @@ FBA_DLL_IMPORT DEVHND fbadasd_device_hndinfo;
 extern DEVHND ctcadpt_device_hndinfo;
 extern DEVHND ctci_device_hndinfo;
 extern DEVHND ctct_device_hndinfo;
+extern DEVHND ctce_device_hndinfo;
 extern DEVHND lcs_device_hndinfo;
 extern DEVHND vmnet_device_hndinfo;
 
