@@ -1,12 +1,6 @@
-/* FAKETAPE.C   (c) Copyright Roger Bowler, 1999-2009                */
+/* FAKETAPE.C   (c) Copyright "Fish" (David B. Trout), 2009          */
 /*              Hercules Tape Device Handler for FAKETAPE            */
 
-/* Original Author: "Fish" (David B. Trout)                          */
-/* Prime Maintainer: "Fish" (David B. Trout)                         */
-/* Secondary Maintainer: Ivan Warren                                 */
-
-// $Id: faketape.c 5587 2009-12-31 15:05:57Z rbowler $
-
 /*-------------------------------------------------------------------*/
 /* This module contains the FAKETAPE emulated tape format support.   */
 /*                                                                   */
@@ -136,14 +130,14 @@ char            pathname[MAX_PATH];     /* file path in host format  */
     hostpath(pathname, dev->filename, sizeof(pathname));
     if(!dev->tdparms.logical_readonly)
     {
-        rc = open (pathname, O_RDWR | O_BINARY);
+        rc = hopen(pathname, O_RDWR | O_BINARY);
     }
 
     /* If file is read-only, attempt to open again */
     if (dev->tdparms.logical_readonly || (rc < 0 && (EROFS == errno || EACCES == errno)))
     {
         dev->readonly = 1;
-        rc = open (pathname, O_RDONLY | O_BINARY);
+        rc = hopen(pathname, O_RDONLY | O_BINARY);
     }
 
     /* Check for successful open */
