@@ -63,7 +63,7 @@
 #define FEATURE_EXTRACT_CPU_TIME                                /*407*/
 #define FEATURE_FETCH_PROTECTION_OVERRIDE
 #define FEATURE_FAST_BCR_SERIALIZATION_FACILITY                 /*810*/
-//#define FEATURE_FLOATING_POINT_EXTENSION_FACILITY               /*810*/
+#define FEATURE_FLOATING_POINT_EXTENSION_FACILITY               /*810*/
 #define FEATURE_FPS_ENHANCEMENT                                 /*DFP*/
 #define FEATURE_FPS_EXTENSIONS
 #define FEATURE_GENERAL_INSTRUCTIONS_EXTENSION_FACILITY         /*208*/
