@@ -1,38 +1,15 @@
-////////////////////////////////////////////////////////////////////////////////////
-//         fthreads.h           Fish's WIN32 version of pthreads
+/*  FTHREADS.H  (c) Copyright "Fish" (David B. Trout), 2001-2012     */
+/*              Fish's WIN32 version of pthreads                     */
+
 ////////////////////////////////////////////////////////////////////////////////////
 // (c) Copyright "Fish" (David B. Trout), 2001-2009. Released under the Q Public License
 // (http://www.hercules-390.org/herclic.html) as modifications to Hercules.
 ////////////////////////////////////////////////////////////////////////////////////
 
-// $Id: fthreads.h 5125 2009-01-23 12:01:44Z bernard $
-//
-// $Log$
-// Revision 1.24  2008/03/25 11:41:31  fish
-// SCSI TAPE MODS part 1: groundwork: non-functional changes:
-// rename some functions, comments, general restructuring, etc.
-// New source modules awstape.c, omatape.c, hettape.c and
-// tapeccws.c added, but not yet used (all will be used in a future
-// commit though when tapedev.c code is eventually split)
-//
-// Revision 1.23  2007/11/30 14:54:32  jmaynard
-// Changed conmicro.cx to hercules-390.org or conmicro.com, as needed.
-//
-// Revision 1.22  2007/06/23 00:04:10  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.21  2007/01/10 09:29:45  fish
-// Fix thread naming that was inadvertently broken by my 12/28 fishhang change that introduced use of _beginthreadex
-//
-// Revision 1.20  2006/12/08 09:43:21  jj
-// Add CVS message log
-//
-
 #ifndef _FTHREADS_H_
 #define _FTHREADS_H_
 
 #include "hercules.h"
-#include "fishhang.h"
 
 #ifndef _FTHREADS_C_
 #ifndef _HUTIL_DLL_
@@ -47,7 +24,31 @@
 ////////////////////////////////////////////////////////////////////////////////////
 // Just a handy macro to have around...
 
-#define  RC(rc)     (errno = rc)
+#define RC(rc)  (rc)
+
+////////////////////////////////////////////////////////////////////////////////////
+
+#define MyInitializeCriticalSection(pCS)                (InitializeCriticalSectionAndSpinCount((CRITICAL_SECTION*)(pCS),3000))
+#define MyEnterCriticalSection(pCS)                     (EnterCriticalSection((CRITICAL_SECTION*)(pCS)))
+#define MyTryEnterCriticalSection(pCS)                  (TryEnterCriticalSection((CRITICAL_SECTION*)(pCS)))
+#define MyLeaveCriticalSection(pCS)                     (LeaveCriticalSection((CRITICAL_SECTION*)(pCS)))
+#define MyDeleteCriticalSection(pCS)                    (DeleteCriticalSection((CRITICAL_SECTION*)(pCS)))
+
+#ifdef _MSVC_
+#define MyCreateThread(sec,stack,start,parm,flags,tid)  ((HANDLE) _beginthreadex((sec),(unsigned)(stack),(start),(parm),(flags),(tid)))
+#define MyExitThread(code)                              (_endthreadex((code)))
+#else // (Cygwin)
+#define MyCreateThread(sec,stack,start,parm,flags,tid)  (CreateThread((sec),(stack),(start),(parm),(flags),(tid)))
+#define MyExitThread(code)                              (ExitThread((code)))
+#endif // _MSVC_
+
+#define MyCreateEvent(sec,man,set,name)                 (CreateEvent((sec),(man),(set),(name)))
+#define MySetEvent(h)                                   (SetEvent((h)))
+#define MyResetEvent(h)                                 (ResetEvent((h)))
+#define MyDeleteEvent(h)                                (CloseHandle((h)))
+#define MyCloseHandle(h)                                (CloseHandle((h)))
+
+#define MyWaitForSingleObject(h,millisecs)              (WaitForSingleObject((h),(millisecs)))
 
 ////////////////////////////////////////////////////////////////////////////////////
 // (need struct timespec for fthread_cond_timedwait)
@@ -166,10 +167,6 @@ int  fthread_attr_getstacksize
 FT_DLL_IMPORT
 int  fthread_join
 (
-#ifdef FISH_HANG
-    const char*     pszFile,
-    const int       nLine,
-#endif
     fthread_t       dwThreadID,
     void**          pExitVal
 );
@@ -189,10 +186,6 @@ int  fthread_detach
 FT_DLL_IMPORT
 int  fthread_create
 (
-#ifdef FISH_HANG
-    const char*  pszFile,
-    const int    nLine,
-#endif
     fthread_t*       pdwThreadID,
     fthread_attr_t*  pThreadAttr,
     PFT_THREAD_FUNC  pfnThreadFunc,
@@ -243,10 +236,6 @@ int  fthread_kill   // FIXME: TODO:
 FT_DLL_IMPORT
 int  fthread_mutex_init
 (
-#ifdef FISH_HANG
-    const char*                 pszFile,
-    const int                   nLine,
-#endif
           fthread_mutex_t*      pFT_MUTEX,
     const fthread_mutexattr_t*  pFT_MUTEX_ATTR
 );
@@ -257,10 +246,6 @@ int  fthread_mutex_init
 FT_DLL_IMPORT
 int  fthread_mutex_destroy
 (
-#ifdef FISH_HANG
-    const char*  pszFile,
-    const int    nLine,
-#endif
     fthread_mutex_t*  pFT_MUTEX
 );
 
@@ -270,10 +255,6 @@ int  fthread_mutex_destroy
 FT_DLL_IMPORT
 int  fthread_mutex_lock
 (
-#ifdef FISH_HANG
-    const char*  pszFile,
-    const int    nLine,
-#endif
     fthread_mutex_t*  pFT_MUTEX
 );
 
@@ -283,10 +264,6 @@ int  fthread_mutex_lock
 FT_DLL_IMPORT
 int  fthread_mutex_trylock
 (
-#ifdef FISH_HANG
-    const char*  pszFile,
-    const int    nLine,
-#endif
     fthread_mutex_t*  pFT_MUTEX
 );
 
@@ -296,10 +273,6 @@ int  fthread_mutex_trylock
 FT_DLL_IMPORT
 int  fthread_mutex_unlock
 (
-#ifdef FISH_HANG
-    const char*  pszFile,
-    const int    nLine,
-#endif
     fthread_mutex_t*  pFT_MUTEX
 );
 
@@ -309,10 +282,6 @@ int  fthread_mutex_unlock
 FT_DLL_IMPORT
 int  fthread_cond_init
 (
-#ifdef FISH_HANG
-    const char*  pszFile,
-    const int    nLine,
-#endif
     fthread_cond_t*  pFT_COND_VAR
 );
 
@@ -322,10 +291,6 @@ int  fthread_cond_init
 FT_DLL_IMPORT
 int  fthread_cond_destroy
 (
-#ifdef FISH_HANG
-    const char*  pszFile,
-    const int    nLine,
-#endif
     fthread_cond_t*  pFT_COND_VAR
 );
 
@@ -335,10 +300,6 @@ int  fthread_cond_destroy
 FT_DLL_IMPORT
 int  fthread_cond_signal
 (
-#ifdef FISH_HANG
-    const char*  pszFile,
-    const int    nLine,
-#endif
     fthread_cond_t*  pFT_COND_VAR
 );
 
@@ -348,10 +309,6 @@ int  fthread_cond_signal
 FT_DLL_IMPORT
 int  fthread_cond_broadcast
 (
-#ifdef FISH_HANG
-    const char*  pszFile,
-    const int    nLine,
-#endif
     fthread_cond_t*  pFT_COND_VAR
 );
 
@@ -361,10 +318,6 @@ int  fthread_cond_broadcast
 FT_DLL_IMPORT
 int  fthread_cond_wait
 (
-#ifdef FISH_HANG
-    const char*  pszFile,
-    const int    nLine,
-#endif
     fthread_cond_t*   pFT_COND_VAR,
     fthread_mutex_t*  pFT_MUTEX
 );
@@ -375,10 +328,6 @@ int  fthread_cond_wait
 FT_DLL_IMPORT
 int  fthread_cond_timedwait
 (
-#ifdef FISH_HANG
-    const char*  pszFile,
-    const int    nLine,
-#endif
     fthread_cond_t*   pFT_COND_VAR,
     fthread_mutex_t*  pFT_MUTEX,
     struct timespec*  pTimeTimeout
