@@ -1,13 +1,5 @@
-/*-------------------------------------------------------------------*/
-/*  HMACROS.H             Hercules macros...                         */
-/*-------------------------------------------------------------------*/
-
-//      This header auto-#included by 'hercules.h'...
-//
-//      The <config.h> header and other required headers are
-//      presumed to have already been #included ahead of it...
-
-// $Id: hmacros.h 5602 2010-01-16 12:15:16Z fish $
+/* HMACROS.H    (c) Copyright Roger Bowler, 1999-2014                */
+/*              Hercules macros                                      */
 
 #ifndef _HMACROS_H
 #define _HMACROS_H
@@ -199,14 +191,7 @@
   #define DEBUG_( _string ) _string
 #endif
 
-#if defined(ENABLE_NLS)
-  #define _(_string) gettext(DEBUG_(_string))
-#else
-  #define _(_string) (DEBUG_(_string))
-  #define N_(_string) (DEBUG_(_string))
-  #define textdomain(_domain)
-  #define bindtextdomain(_package, _directory)
-#endif
+#define _(_string) (DEBUG_(_string))
 
 #if defined(DEBUG) || defined(_DEBUG)
 
@@ -280,7 +265,7 @@ typedef U32  (*s390_trace_br_func) (int amode,  U32 ia, REGS *regs);
 typedef U64  (*z900_trace_br_func) (int amode,  U64 ia, REGS *regs);
 
 /*-------------------------------------------------------------------*/
-/* compiler optimization hints         (for performance)             */
+/* Compiler optimization hints (for performance)                     */
 /*-------------------------------------------------------------------*/
 
 #undef likely
@@ -288,8 +273,8 @@ typedef U64  (*z900_trace_br_func) (int amode,  U64 ia, REGS *regs);
 
 #ifdef _MSVC_
 
-  #define likely(_c)      ( (_c) ? ( __assume((_c)), 1 ) :                    0   )
-  #define unlikely(_c)    ( (_c) ?                   1   : ( __assume(!(_c)), 0 ) )
+  #define likely(_c)      ( (_c) ? ( __assume((_c)), 1 ) : 0 )
+  #define unlikely(_c)    ( (_c) ? 1 : ( __assume(!(_c)), 0 ) )
 
 #else // !_MSVC_
 
@@ -541,7 +526,6 @@ typedef U64  (*z900_trace_br_func) (int amode,  U64 ia, REGS *regs);
     ( (_dev)->hnd->immed && (_dev)->hnd->immed[(_dev)->code]) \
     || ( (_dev)->immed      && (_dev)->immed[(_dev)->code]) \
     || IS_CCW_NOP((_dev)->code) \
-    || IS_CCW_SET_EXTENDED((_dev)->code) \
   )
 
 /*-------------------------------------------------------------------*/
@@ -592,17 +576,6 @@ typedef U64  (*z900_trace_br_func) (int amode,  U64 ia, REGS *regs);
 /* Perform standard utility initialization                           */
 /*-------------------------------------------------------------------*/
 
-#if !defined(ENABLE_NLS)
-  #define INITIALIZE_NLS()
-#else
-  #define INITIALIZE_NLS() \
-  do { \
-    setlocale(LC_ALL, ""); \
-    bindtextdomain(PACKAGE, HERC_LOCALEDIR); \
-    textdomain(PACKAGE); \
-  } while (0)
-#endif
-
 #if !defined(EXTERNALGUI)
   #define INITIALIZE_EXTERNAL_GUI()
 #else
@@ -621,7 +594,6 @@ typedef U64  (*z900_trace_br_func) (int amode,  U64 ia, REGS *regs);
 #define INITIALIZE_UTILITY(name) \
   do { \
     SET_THREAD_NAME(name); \
-    INITIALIZE_NLS(); \
     INITIALIZE_EXTERNAL_GUI(); \
     memset (&sysblk, 0, sizeof(SYSBLK)); \
     initialize_detach_attr (DETACHED); \
