@@ -689,11 +689,11 @@ void socket_keepalive( int sfd, int idle_time, int probe_interval,
 
   #if defined(TCP_KEEPALIVE)
     optval = idle_time;
-    rc = setsockopt(sfd, IPPROTO_TCP, TCP_KEEPALIVE, &optval, sizeof(optval)); 
+    rc = setsockopt(sfd, IPPROTO_TCP, TCP_KEEPALIVE, &optval, sizeof(optval));
     if (rc) logmsg("HHCUT002I TCP_KEEPALIVE rc=%d %s\n", rc, strerror(errno));
   #elif defined(TCP_KEEPIDLE)
     optval = idle_time;
-    rc = setsockopt(sfd, SOL_TCP, TCP_KEEPIDLE, &optval, sizeof(optval)); 
+    rc = setsockopt(sfd, IPPROTO_TCP, TCP_KEEPIDLE, &optval, sizeof(optval));
     if (rc) logmsg("HHCUT003I TCP_KEEPIDLE rc=%d %s\n", rc, strerror(errno));
   #else
     UNREFERENCED(idle_time);
@@ -701,7 +701,7 @@ void socket_keepalive( int sfd, int idle_time, int probe_interval,
 
   #if defined(TCP_KEEPINTVL)
     optval = probe_interval;
-    rc = setsockopt(sfd, SOL_TCP, TCP_KEEPINTVL, &optval, sizeof(optval)); 
+    rc = setsockopt(sfd, IPPROTO_TCP, TCP_KEEPINTVL, &optval, sizeof(optval));
     if (rc) logmsg("HHCUT004I TCP_KEEPINTVL rc=%d %s\n", rc, strerror(errno));
   #else
     UNREFERENCED(probe_interval);
@@ -709,7 +709,7 @@ void socket_keepalive( int sfd, int idle_time, int probe_interval,
 
   #if defined(TCP_KEEPCNT)
     optval = probe_count;
-    rc = setsockopt(sfd, SOL_TCP, TCP_KEEPCNT, &optval, sizeof(optval)); 
+    rc = setsockopt(sfd, IPPROTO_TCP, TCP_KEEPCNT, &optval, sizeof(optval));
     if (rc) logmsg("HHCUT005I TCP_KEEPCNT rc=%d %s\n", rc, strerror(errno));
   #else
     UNREFERENCED(probe_count);
