@@ -1,8 +1,6 @@
 /* HTTPSERV.C   (c)Copyright Jan Jaeger, 2002-2009                   */
 /*              HTTP Server                                          */
 
-// $Id: httpserv.c 5146 2009-02-01 20:09:49Z rbowler $
-
 /* This file contains all code required for the HTTP server,         */
 /* when the http_server thread is started it will listen on          */
 /* the HTTP port specified on the HTTPPORT config statement.         */
@@ -29,35 +27,6 @@
 /*                                                                   */
 /*                                           Jan Jaeger - 28/03/2002 */
 
-// $Log$
-// Revision 1.80  2009/01/23 12:51:42  bernard
-// copyright notice
-//
-// Revision 1.79  2009/01/15 17:36:43  jj
-// Change http server startup
-//
-// Revision 1.78  2008/11/04 05:56:31  fish
-// Put ensure consistent create_thread ATTR usage change back in
-//
-// Revision 1.77  2008/11/03 15:31:54  rbowler
-// Back out consistent create_thread ATTR modification
-//
-// Revision 1.76  2008/10/18 09:32:21  fish
-// Ensure consistent create_thread ATTR usage
-//
-// Revision 1.75  2008/05/28 16:39:44  fish
-// (use R_OK constant)
-//
-// Revision 1.74  2007/06/23 00:04:12  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.73  2006/12/31 23:49:00  fish
-// Use "access()" to validate http_server root directory and not "chdir()"!!
-//
-// Revision 1.72  2006/12/08 09:43:28  jj
-// Add CVS message log
-//
-
 #include "hstdinc.h"
 
 #define _HTTPSERV_C_
@@ -302,7 +271,7 @@ static char *http_unescape(char *buffer)
 
         *pointer = (highnibble<<4) | lownibble;
 
-        memcpy(pointer+1, pointer+3, strlen(pointer+3)+1);
+        memmove(pointer+1, pointer+3, strlen(pointer+3)+1);
 
         pointer++;
     }
@@ -468,7 +437,7 @@ static void http_download(WEBBLK *webblk, char *filename)
         http_error(webblk, "404 File Not Found","",
                            "The requested file is not a regular file");
 
-    fd = open(fullname,O_RDONLY|O_BINARY,0);
+    fd = hopen(fullname,O_RDONLY|O_BINARY,0);
     if (fd == -1)
         http_error(webblk, "404 File Not Found","",
                            strerror(errno));
