@@ -1,9 +1,8 @@
-/* PFPO.C       (c) Copyright Roger Bowler, 2009                     */
-/*              (c) Copyright Bernard van der Helm, 2009             */
-/*              Noordwijkerhout, The Netherlands                     */ 
+/* PFPO.C       (c) Copyright Roger Bowler, 2007                     */
 /*              Perform Floating Point Operation instruction         */
 
-// $Id: pfpo.c 5453 2009-08-14 07:18:18Z bernard $
+/*              (c) Copyright Bernard van der Helm, 2009             */
+/*              Noordwijkerhout, The Netherlands                     */
 
 /*-------------------------------------------------------------------*/
 /* This module implements the Perform Floating Point Operation       */
@@ -48,13 +47,14 @@ typedef struct
   unsigned class;
   char str[256];
 }
-FLOAT;
+PFPO_FLOAT;
 
+#if 0 // PFPO not yet implemented
 /*-------------------------------------------------------------------*/
-/* Binairy floating point routines                                   */
+/* Binary floating point routines                                    */
 /*-------------------------------------------------------------------*/
 
-void BFPshortGet(FLOAT *f, U32 r)
+static void BFPshortGet(PFPO_FLOAT *f, U32 r)
 {
   unsigned exp;
   U32 frac;
@@ -138,7 +138,7 @@ void BFPshortGet(FLOAT *f, U32 r)
   }
 }
 
-void BFPlongGet(FLOAT *f, U64 r)
+static void BFPlongGet(PFPO_FLOAT *f, U64 r)
 {
   unsigned exp;
   U64 frac;
@@ -222,7 +222,7 @@ void BFPlongGet(FLOAT *f, U64 r)
   }
 }
 
-void BFPextGet(FLOAT *f, U64 h, U64 l)
+static void BFPextGet(PFPO_FLOAT *f, U64 h, U64 l)
 {
   unsigned exp;
   U64 frach;
@@ -321,9 +321,8 @@ void BFPextGet(FLOAT *f, U64 h, U64 l)
 /* Decimal floating point routines                                   */
 /*-------------------------------------------------------------------*/
 
-void DFPshortGet(FLOAT *f, U32 r)
+static void DFPshortGet(PFPO_FLOAT *f, U32 r)
 {
-  unsigned class;
   decimal32 dec32;
   unsigned i;
   U32 temp;
@@ -351,9 +350,8 @@ void DFPshortGet(FLOAT *f, U32 r)
     f->class = CLASS_NORMAL;
 }
 
-void DFPlongGet(FLOAT *f, U64 r)
+static void DFPlongGet(PFPO_FLOAT *f, U64 r)
 {
-  unsigned class;
   decimal64 dec64;
   unsigned i;
   U64 temp;
@@ -381,9 +379,8 @@ void DFPlongGet(FLOAT *f, U64 r)
     f->class = CLASS_NORMAL;
 }
 
-void DFPextGet(FLOAT *f, U64 h, U64 l)
+static void DFPextGet(PFPO_FLOAT *f, U64 h, U64 l)
 {
-  unsigned class;
   decimal128 dec128;
   unsigned i;
   U64 temph;
@@ -415,12 +412,12 @@ void DFPextGet(FLOAT *f, U64 h, U64 l)
   else
     f->class = CLASS_NORMAL;
 }
- 
+
 /*-------------------------------------------------------------------*/
 /* Hexadecimal floating point routines                               */
 /*-------------------------------------------------------------------*/
 
-void HFPshortGet(FLOAT *f, U32 r)
+static void HFPshortGet(PFPO_FLOAT *f, U32 r)
 {
   unsigned exp;
   U32 frac;
@@ -461,9 +458,8 @@ void HFPshortGet(FLOAT *f, U32 r)
   }
 }
 
-void HFPlongGet(FLOAT *f, U64 r)
+static void HFPlongGet(PFPO_FLOAT *f, U64 r)
 {
-  unsigned class;
   unsigned exp;
   U64 frac;
   unsigned i;
@@ -480,7 +476,7 @@ void HFPlongGet(FLOAT *f, U64 r)
     f->class = CLASS_NORMAL;
 
   f->base = 16;
-  switch(class)
+  switch(f->class)
   {
     case CLASS_ZERO:
       strcpy(f->str, "0");
@@ -494,7 +490,7 @@ void HFPlongGet(FLOAT *f, U64 r)
       mask = 0x00ff000000000000;
       for(i = 0; i < 7; i++)
       {
-        sprintf(&f->str[strlen(f->str)], "%02x", (r & mask) >> (48 - (i * 8)));
+        sprintf(&f->str[strlen(f->str)], "%02"I64_FMT"x", (r & mask) >> (48 - (i * 8)));
         mask >>= 8;
       }
       strcat(f->str, "@");
@@ -503,9 +499,8 @@ void HFPlongGet(FLOAT *f, U64 r)
   }
 }
 
-void HFPextGet(FLOAT *f, U64 h, U64 l)
+static void HFPextGet(PFPO_FLOAT *f, U64 h, U64 l)
 {
-  unsigned class;
   unsigned exp;
   U64 frach;
   U64 fracl;
@@ -538,13 +533,13 @@ void HFPextGet(FLOAT *f, U64 h, U64 l)
       mask = 0x00ff000000000000;
       for(i = 0; i < 7; i++)
       {
-        sprintf(&f->str[strlen(f->str)], "%02x", (h & mask) >> (48 - (i * 8)));
+        sprintf(&f->str[strlen(f->str)], "%02"I64_FMT"x", (h & mask) >> (48 - (i * 8)));
         mask >>= 8;
       }
       mask = 0x00ff000000000000;
       for(i = 0; i < 7; i++)
       {
-        sprintf(&f->str[strlen(f->str)], "%02x", (l & mask) >> (48 - (i * 8)));
+        sprintf(&f->str[strlen(f->str)], "%02"I64_FMT"x", (l & mask) >> (48 - (i * 8)));
         mask >>= 8;
       }
       strcat(f->str, "@");
@@ -552,6 +547,7 @@ void HFPextGet(FLOAT *f, U64 h, U64 l)
       break;
   }
 }
+#endif // PFPO not yet implemented
 
 /*-------------------------------------------------------------------*/
 /* 010A PFPO  - Perform Floating Point Operation                 [E] */
