@@ -1,16 +1,16 @@
-/* TAPEDEV.C    (c) Copyright Roger Bowler, 1999-2007                */
-/*              ESA/390 Tape Device Handler                          */
+/* TAPEDEV.C    (c) Copyright Roger Bowler, 1999-2009                */
+/*              Hercules Tape Device Handler                         */
 
 /* Original Author: Roger Bowler                                     */
 /* Prime Maintainer: Ivan Warren                                     */
 /* Secondary Maintainer: "Fish" (David B. Trout)                     */
 
-// $Id: tapedev.c,v 1.151 2008/11/01 20:40:09 fish Exp $
-
 /*-------------------------------------------------------------------*/
 /* This module contains device handling functions for emulated       */
 /* magnetic tape devices for the Hercules ESA/390 emulator.          */
 /*-------------------------------------------------------------------*/
+/* Messages issued by the TAPEDEV.C module are prefixed HHCTA0nn     */
+/* CCW processing functions have been moved to module TAPECCW.C      */
 /*                                                                   */
 /* Five emulated tape formats are supported:                         */
 /*                                                                   */
@@ -24,7 +24,6 @@
 /*                                                                   */
 /*              Support for AWSTAPE is in the "AWSTAPE.C" member.    */
 /*                                                                   */
-/*                                                                   */
 /* 2. OMATAPE   This is the Optical Media Attach device format.      */
 /*              Each physical file on the tape is represented by     */
 /*              a separate flat file.  The collection of files that  */
@@ -48,7 +47,6 @@
 /*                                                                   */
 /*              OMATAPE tape Support is in the "OMATAPE.C" member.   */
 /*                                                                   */
-/*                                                                   */
 /* 3. SCSITAPE  This format allows reading and writing of 4mm or     */
 /*              8mm DAT tape, 9-track open-reel tape, or 3480-type   */
 /*              cartridge on an appropriate SCSI-attached drive.     */
@@ -60,7 +58,6 @@
 /*                                                                   */
 /*              SCSI tape Support is in the "SCSITAPE.C" member.     */
 /*                                                                   */
-/*                                                                   */
 /* 4. HET       This format is based on the AWSTAPE format but has   */
 /*              been extended to support compression.  Since the     */
 /*              basic file format has remained the same, AWSTAPEs    */
@@ -68,7 +65,6 @@
 /*                                                                   */
 /*              Support for HET is in the "HETTAPE.C" member.        */
 /*                                                                   */
-/*                                                                   */
 /* 5. FAKETAPE  This is the format used by Fundamental Software      */
 /*              on their FLEX-ES systems. It it similar to the AWS   */
 /*              format. The entire tape is contained in a single     */
@@ -108,163 +104,12 @@
 /* GA32-0127 IBM 3490E Hardware Reference                            */
 /* GC35-0152 EREP Release 3.5.0 Reference                            */
 /* SA22-7204 ESA/390 Common I/O-Device Commands                      */
-/* Flex FakeTape format (http://preview.tinyurl.com/67rgnp)          */
-/*-------------------------------------------------------------------*/
-
-// $Log: tapedev.c,v $
-// Revision 1.151  2008/11/01 20:40:09  fish
-// Fix tapedev gettapetype_bydata function for HET/AWS tapes
-// that start with a tapemark -- Liam Cold [liamcold@yahoo.com]
-//
-// Revision 1.150  2008/09/10 02:06:48  fish
-// Add test for OMA file "@TDF" signature
-// to 'gettapetype_bydata' function
-//
-// Revision 1.149  2008/07/21 22:13:29  rbowler
-// tapedev.c(964) : warning C4101: 'i' : unreferenced local variable
-//
-// Revision 1.148  2008/07/08 05:35:51  fish
-// AUTOMOUNT redesign: support +allowed/-disallowed dirs
-// and create associated 'automount' panel command - Fish
-//
-// Revision 1.147  2008/06/22 05:54:30  fish
-// Fix print-formatting issue (mostly in tape modules)
-// that can sometimes, in certain circumstances,
-// cause herc to crash.  (%8.8lx --> I32_FMTX, etc)
-//
-// Revision 1.146  2008/05/28 16:46:29  fish
-// Misleading VTAPE support renamed to AUTOMOUNT instead and fixed and enhanced so that it actually WORKS now.
-//
-// Revision 1.145  2008/05/25 06:36:43  fish
-// VTAPE automount support (0x4B + 0xE4)
-//
-// Revision 1.144  2008/05/23 20:37:05  fish
-// Fix "presuming" message in gettapetype function to use implemented default shortdesc in message.
-//
-// Revision 1.143  2008/05/22 21:17:29  fish
-// Tape file extension neutrality support
-//
-// Revision 1.142  2008/05/22 20:15:12  fish
-// Correct --blkid-24 typo (SHOULD be --blkid-22, not 24)
-//
-// Revision 1.141  2008/05/22 19:25:58  fish
-// Flex FakeTape support
-//
-// Revision 1.140  2008/04/05 02:36:00  fish
-// Wrap GENTMH_SCSI_ACTION... case in 'generic_tmhcall' function
-// with #ifdef OPTION_SCSI_TAPE to fix undefined symbol error
-// on non-scsi-tape build platforms
-//
-// Revision 1.139  2008/03/30 02:51:34  fish
-// Fix SCSI tape EOV (end of volume) processing
-//
-// Revision 1.138  2008/03/29 08:36:46  fish
-// More complete/extensive 3490/3590 tape support
-//
-// Revision 1.137  2008/03/28 02:09:42  fish
-// Add --blkid-24 option support, poserror flag renamed to fenced,
-// added 'generic', 'readblkid' and 'locateblk' tape media handler
-// call vectors.
-//
-// Revision 1.136  2008/03/27 07:14:17  fish
-// SCSI MODS: groundwork: part 3: final shuffling around.
-// Moved functions from one module to another and resequenced
-// functions within each. NO CODE WAS ACTUALLY CHANGED.
-// Next commit will begin the actual changes.
-//
-// Revision 1.135  2008/03/26 07:23:51  fish
-// SCSI MODS part 2: split tapedev.c: aws, het, oma processing moved
-// to separate modules, CCW processing moved to separate module.
-//
-// Revision 1.134  2008/03/25 11:41:31  fish
-// SCSI TAPE MODS part 1: groundwork: non-functional changes:
-// rename some functions, comments, general restructuring, etc.
-// New source modules awstape.c, omatape.c, hettape.c and
-// tapeccws.c added, but not yet used (all will be used in a future
-// commit though when tapedev.c code is eventually split)
-//
-// Revision 1.133  2008/03/13 01:44:17  kleonard
-// Fix residual read-only setting for tape device
-//
-// Revision 1.132  2008/03/04 01:10:29  ivan
-// Add LEGACYSENSEID config statement to allow X'E4' Sense ID on devices
-// that originally didn't support it. Defaults to off for compatibility reasons
-//
-// Revision 1.131  2008/03/04 00:25:25  ivan
-// Ooops.. finger check on 8809 case for numdevid.. Thanks Roger !
-//
-// Revision 1.130  2008/03/02 12:00:04  ivan
-// Re-disable Sense ID on 3410, 3420, 8809 : report came in that it breaks MTS
-//
-// Revision 1.129  2007/12/14 17:48:52  rbowler
-// Enable SENSE ID CCW for 2703,3410,3420
-//
-// Revision 1.128  2007/11/29 03:36:40  fish
-// Re-sequence CCW opcode 'case' statements to be in ascending order.
-// COSMETIC CHANGE ONLY. NO ACTUAL LOGIC WAS CHANGED.
-//
-// Revision 1.127  2007/11/13 15:10:52  rbowler
-// fsb_awstape support for segmented blocks
-//
-// Revision 1.126  2007/11/11 20:46:50  rbowler
-// read_awstape support for segmented blocks
-//
-// Revision 1.125  2007/11/09 14:59:34  rbowler
-// Move misplaced comment and restore original programming style
-//
-// Revision 1.124  2007/11/02 16:04:15  jmaynard
-// Removing redundant #if !(defined OPTION_SCSI_TAPE).
-//
-// Revision 1.123  2007/09/01 06:32:24  fish
-// Surround 3590 SCSI test w/#ifdef (OPTION_SCSI_TAPE)
-//
-// Revision 1.122  2007/08/26 14:37:17  fish
-// Fix missed unfixed 31 Aug 2006 non-SCSI tape Locate bug
-//
-// Revision 1.121  2007/07/24 23:06:32  fish
-// Force command-reject for 3590 Medium Sense and Mode Sense
-//
-// Revision 1.120  2007/07/24 22:54:49  fish
-// (comment changes only)
-//
-// Revision 1.119  2007/07/24 22:46:09  fish
-// Default to --blkid-32 and --no-erg for 3590 SCSI
-//
-// Revision 1.118  2007/07/24 22:36:33  fish
-// Fix tape Synchronize CCW (x'43') to do actual commit
-//
-// Revision 1.117  2007/07/24 21:57:29  fish
-// Fix Win32 SCSI tape "Locate" and "ReadBlockId" SNAFU
-//
-// Revision 1.116  2007/06/23 00:04:18  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.115  2007/04/06 15:40:25  fish
-// Fix Locate Block & Read BlockId for SCSI tape broken by 31 Aug 2006 preliminary-3590-support change
-//
-// Revision 1.114  2007/02/25 21:10:44  fish
-// Fix het_locate to continue on tapemark
-//
-// Revision 1.113  2007/02/03 18:58:06  gsmith
-// Fix MVT tape CMDREJ error
-//
-// Revision 1.112  2006/12/28 03:04:17  fish
-// PR# tape/100: Fix crash in "open_omatape()" in tapedev.c if bad filespec entered in OMA (TDF)  file
-//
-// Revision 1.111  2006/12/11 17:25:59  rbowler
-// Change locblock from long to U32 to correspond with dev->blockid
-//
-// Revision 1.110  2006/12/08 09:43:30  jj
-// Add CVS message log
-//
 /*-------------------------------------------------------------------*/
 
 #include "hstdinc.h"
 #include "hercules.h"  /* need Hercules control blocks               */
 #include "tapedev.h"   /* Main tape handler header file              */
 
-/*-------------------------------------------------------------------*/
-
 //#define  ENABLE_TRACING_STMTS     // (Fish: DEBUGGING)
 
 #ifdef ENABLE_TRACING_STMTS
@@ -649,10 +494,43 @@ int tapedev_init_handler (DEVBLK *dev, int argc, char *argv[])
 {
 int             rc;
 DEVINITTAB*     pDevInitTab;
+int             attn = 0;
+
+    /* Set flag so attention will be raised for re-init */
+    if(dev->devtype)
+    {
+        attn = 1;
+    }
 
     /* Close current tape */
     if(dev->fd>=0)
     {
+        /* Prevent accidental re-init'ing of already loaded tape drives */
+        if (sysblk.nomountedtapereinit)
+        {
+            char*  devclass;
+
+            tapedev_query_device(dev, &devclass, 0, NULL);
+
+            if (1
+                && strcmp(devclass,"TAPE") == 0
+                && (0
+                    || TAPEDEVT_SCSITAPE == dev->tapedevt
+                    || (argc >= 3 && strcmp(argv[2], TAPE_UNLOADED) != 0)
+                   )
+            )
+            {
+                ASSERT( dev->tmh && dev->tmh->tapeloaded );
+                if (dev->tmh->tapeloaded( dev, NULL, 0 ))
+                {
+                    release_lock (&dev->lock);
+                    logmsg(_("HHCPN183E Reinit rejected for drive %u:%4.4X; drive not empty\n"),
+                        SSID_TO_LCSS(dev->ssid), dev->devnum);
+                    return -1;
+                }
+            }
+        }
+
         dev->tmh->close(dev);
         dev->fd=-1;
     }
@@ -781,8 +659,7 @@ DEVINITTAB*     pDevInitTab;
 
     /* Initialize SCSI tape control fields */
 #if defined(OPTION_SCSI_TAPE)
-    dev->sstat               = GMT_DR_OPEN(-1);
-    dev->stape_getstat_sstat = GMT_DR_OPEN(-1);
+    dev->sstat = GMT_DR_OPEN(-1);
 #endif
 
     /* Clear the DPA */
@@ -794,9 +671,6 @@ DEVINITTAB*     pDevInitTab;
        a single buffer before passing data to the device handler */
     dev->cdwmerge = 1;
 
-    /* Tape is a syncio type 2 device */
-    dev->syncio = 2;
-
     /* ISW */
     /* Build a 'clear' sense */
     memset (dev->sense, 0, sizeof(dev->sense));
@@ -844,6 +718,19 @@ DEVINITTAB*     pDevInitTab;
     if (dev->devchar[8] & 0x08)     // SIC supported?
         dev->SIC_supported = 1;     // remember that fact
 
+    if (dev->tapedevt == TAPEDEVT_SCSITAPE)
+        dev->syncio = 0;  // (SCSI i/o too slow; causes Machine checks)
+    else
+        dev->syncio = 2;  // (aws/het/etc are fast; syncio likely safe)
+
+    /* Make attention pending if necessary */
+    if(attn)
+    {
+        release_lock (&dev->lock);
+        device_attention (dev, CSW_DE);
+        obtain_lock (&dev->lock);
+    }
+
     return rc;
 
 } /* end function tapedev_init_handler */
@@ -948,7 +835,7 @@ struct  tape_format_entry   fmttab   [] =   /*    (table itself)     */
 #undef   SCSITAPE_FMTENTRY
 #define  SCSITAPE_FMTENTRY  5
     {
-        "^\\\\\\\\\\.\\\\Tape[0-9]",
+        "^\\\\\\\\\\.\\\\\\w",
         TAPEDEVT_SCSITAPE,
         &tmh_scsi,
         "SCSI attached tape drive",
@@ -979,13 +866,13 @@ int gettapetype_byname (DEVBLK *dev)
 
 #if defined(HAVE_REGEX_H) || defined(HAVE_PCRE)
 
-    for (i=0; i < arraysize( fmttab ); i++)
+    for (i=0; i < (int)arraysize( fmttab ); i++)
     {
         rc = regcomp (&regwrk, fmttab[i].fmtreg, REG_ICASE);
         if (rc < 0)
         {
             regerror (rc, &regwrk, errbfr, 1024);
-            logmsg (_("HHCTA999E Device %4.4X: Unable to determine tape format type for %s: Internal error: Regcomp error %s on index %d\n"),
+            logmsg (_("HHCTA001E %4.4X: Unable to determine tape format type for %s: Internal error: Regcomp error %s on index %d\n"),
                 dev->devnum, dev->filename, errbfr, i);
             return -1;
         }
@@ -995,7 +882,7 @@ int gettapetype_byname (DEVBLK *dev)
         {
             regerror (rc, &regwrk, errbfr, 1024);
             regfree ( &regwrk );
-            logmsg (_("HHCTA999E Device %4.4X: Unable to determine tape format type for %s: Internal error: Regexec error %s on index %d\n"),
+            logmsg (_("HHCTA002E %4.4X: Unable to determine tape format type for %s: Internal error: Regexec error %s on index %d\n"),
                 dev->devnum, dev->filename, errbfr, i);
             return -1;
         }
@@ -1057,9 +944,8 @@ int gettapetype_byname (DEVBLK *dev)
     }
 #if defined(_MSVC_)
     if (1
-        && strncasecmp(dev->filename, "\\\\.\\Tape", 8) == 0
-        && isdigit(*(dev->filename+8))
-        &&  0  ==  *(dev->filename+9)
+        && strncasecmp(dev->filename, "\\\\.\\", 4) == 0
+        &&           *(dev->filename        +    4) != 0
     )
     {
         return SCSITAPE_FMTENTRY;
@@ -1085,7 +971,7 @@ int gettapetype_bydata (DEVBLK *dev)
 
     /* Try to determine the type based on actual file contents */
     hostpath( pathname, dev->filename, sizeof(pathname) );
-    rc = open ( pathname, O_RDONLY | O_BINARY );
+    rc = hopen( pathname, O_RDONLY | O_BINARY );
     if (rc >= 0)
     {
         BYTE hdr[6];                    /* block header i/o buffer   */
@@ -1105,17 +991,23 @@ int gettapetype_bydata (DEVBLK *dev)
             if (memcmp(hdr, "@TDF", 4) == 0)
                 return OMATAPE_FMTENTRY;
 
-            if (hdr[0] == 0x30 && hdr[1] == 0x30 && hdr[2] == 0x30 && hdr[3] == 0x30)
-                return FAKETAPE_FMTENTRY;
+            if (1
+                && hdr[0] == 0x30           /* "ASCII"-zero len prev block? */
+                && hdr[1] == 0x30
+                && hdr[2] == 0x30
+                && hdr[3] == 0x30
+            )
+                return FAKETAPE_FMTENTRY;   /* Then obviously Flex FakeTape */
 
-            if (hdr[2] == 0 && hdr[3] == 0)
+            if (hdr[2] == 0 && hdr[3] == 0)             /* 0 len prev blk?  */
             {
-                if (hdr[4] & 0x40)  /* TapeMark? */
-                    return -1;      /* Unable to determine */
-                if (hdr[4] & 0x03)  /* ZLIB and/or BZIP2 compressed? */
-                    return HETTAPE_FMTENTRY;
+                if (hdr[4] & HETHDR_FLAGS1_TAPEMARK)    /* If tapemark then */
+                    return -1;                          /* can't tell type. */
+                if (hdr[4] & HETHDR_FLAGS1_COMPRESS ||  /* ZLIB or BZIP2 or */
+                    hdr[5] & HETHDR_FLAGS2_COMPRESS)    /* Bus-Tech ZLIB?   */
+                    return HETTAPE_FMTENTRY;            /* Then HET format. */
                 else
-                    return AWSTAPE_FMTENTRY;    /* (default) */
+                    return AWSTAPE_FMTENTRY;            /* Else default AWS */
             }
         }
     }
@@ -1132,33 +1024,29 @@ int gettapetype_bydata (DEVBLK *dev)
 int gettapetype (DEVBLK *dev, char **short_descr)
 {
     char*       descr;                  /* Device descr from fmttab  */
-    int         i = -1;                 /* fmttab entry#             */
+    int         i;                      /* fmttab entry#             */
+
+    i = gettapetype_byname( dev );      /* Get type based on name    */
 
-    /* Try to determine device type by actual file contents first,
-       but only if this isn't a SCSI tape device. (Thus we need to
-       check by name first to determine if it's a SCSI device) */
 #if defined(OPTION_SCSI_TAPE)
-    i = gettapetype_byname( dev );          // (check if this is a SCSI)
-
-    if (i != SCSITAPE_FMTENTRY)             // (if not, then check by..
+    if (i != SCSITAPE_FMTENTRY)         /* If not SCSI tape...       */
 #endif
     {
-        int i2 = gettapetype_bydata( dev ); // ..reading the file data)
-        if (i2 >= 0)                        // (if that worked..)
-            i = i2;                         // (..then we know the type)
+        int i2 = gettapetype_bydata( dev ); // Get type based on data..
+
+        if (i2 >= 0 &&                      // If valid type by data, AND
+           (i2 != AWSTAPE_FMTENTRY ||       // *not* AWS by data (or if it
+            i  != HETTAPE_FMTENTRY)         // is, if it's not HET by name)..
+        )
+            i = i2;                         // ..Use type based on data.
     }
 
-    /* If we couldn't determine the device type based on the file's
-       contents, then try again but this time based on its filename */
-    if (i < 0)
-        i = gettapetype_byname( dev );
-
-    /* If still unknown, use a reasonable default value */
+    /* If file type still unknown, use a reasonable default value... */
     if (i < 0)
     {
         i = DEFAULT_FMTENTRY;
         if (strcmp (dev->filename, TAPE_UNLOADED) != 0)
-            logmsg (_("HHCTA999W Device %4.4X: Unable to determine tape format type for %s; presuming %s.\n"),
+            logmsg (_("HHCTA003W %4.4X: Unable to determine tape format type for %s; presuming %s.\n"),
                  dev->devnum, dev->filename, fmttab[i].short_descr );
     }
 
@@ -1168,7 +1056,7 @@ int gettapetype (DEVBLK *dev, char **short_descr)
     *short_descr  = fmttab[i].short_descr;
 
     if (strcmp (dev->filename, TAPE_UNLOADED) != 0)
-        logmsg (_("HHCTA998I Device %4.4X: %s is a %s\n"),
+        logmsg (_("HHCTA004I %4.4X: %s is a %s\n"),
             dev->devnum, dev->filename, descr);
 
     return 0;   // (success)
@@ -1304,8 +1192,7 @@ int  mountnewtape ( DEVBLK *dev, int argc, char **argv )
     /* Initialize device dependent fields */
     dev->fd                = -1;
 #if defined(OPTION_SCSI_TAPE)
-    dev->sstat               = GMT_DR_OPEN(-1);
-    dev->stape_getstat_sstat = GMT_DR_OPEN(-1);
+    dev->sstat             = GMT_DR_OPEN(-1);
 #endif
     dev->omadesc           = NULL;
     dev->omafiles          = 0;
@@ -1338,7 +1225,7 @@ int  mountnewtape ( DEVBLK *dev, int argc, char **argv )
     }
 #endif
 
-#define  HHCTA078E()  logmsg (_("HHCTA078E Device %4.4X: option '%s' not valid for %s\n"), \
+#define  HHCTA078E()  logmsg (_("HHCTA078E %4.4X: option '%s' not valid for %s\n"), \
                               dev->devnum, argv[i], short_descr)
 
     /* Process remaining options */
@@ -1349,7 +1236,7 @@ int  mountnewtape ( DEVBLK *dev, int argc, char **argv )
         switch (parser (&ptab[0], argv[i], &res))
         {
         case TDPARM_NONE:
-            logmsg (_("HHCTA067E Device %4.4X: option '%s' unrecognized\n"),
+            logmsg (_("HHCTA067E %4.4X: option '%s' unrecognized\n"),
                 dev->devnum, argv[i]);
             optrc = -1;
             break;
@@ -1388,7 +1275,7 @@ int  mountnewtape ( DEVBLK *dev, int argc, char **argv )
             }
             if (res.num < HETMIN_METHOD || res.num > HETMAX_METHOD)
             {
-                logmsg(_("HHCTA068E Device %4.4X: option '%s': method must be within %u-%u\n"),
+                logmsg(_("HHCTA068E %4.4X: option '%s': method must be within %u-%u\n"),
                     dev->devnum, argv[i], HETMIN_METHOD, HETMAX_METHOD);
                 optrc = -1;
                 break;
@@ -1406,7 +1293,7 @@ int  mountnewtape ( DEVBLK *dev, int argc, char **argv )
             }
             if (res.num < HETMIN_LEVEL || res.num > HETMAX_LEVEL)
             {
-                logmsg(_("HHCTA069E Device %4.4X: option '%s': level must be within %u-%u\n"),
+                logmsg(_("HHCTA069E %4.4X: option '%s': level must be within %u-%u\n"),
                     dev->devnum, argv[i], HETMIN_LEVEL, HETMAX_LEVEL);
                 optrc = -1;
                 break;
@@ -1424,7 +1311,7 @@ int  mountnewtape ( DEVBLK *dev, int argc, char **argv )
             }
             if (res.num < HETMIN_CHUNKSIZE || res.num > HETMAX_CHUNKSIZE)
             {
-                logmsg (_("HHCTA070E Device %4.4X: option '%s': chunksize must be within %u-%u\n"),
+                logmsg (_("HHCTA070E %4.4X: option '%s': chunksize must be within %u-%u\n"),
                     dev->devnum, argv[i], HETMIN_CHUNKSIZE, HETMAX_CHUNKSIZE);
                 optrc = -1;
                 break;
@@ -1542,7 +1429,7 @@ int  mountnewtape ( DEVBLK *dev, int argc, char **argv )
 #endif /* defined(OPTION_SCSI_TAPE) */
 
         default:
-            logmsg(_("HHCTA071E Device %4.4X: option '%s': parse error\n"),
+            logmsg(_("HHCTA071E %4.4X: option '%s': parse error\n"),
                 dev->devnum, argv[i]);
             optrc = -1;
             break;
@@ -1552,7 +1439,7 @@ int  mountnewtape ( DEVBLK *dev, int argc, char **argv )
         if (optrc < 0)
             rc = -1;
         else
-            logmsg (_("HHCTA066I Device %4.4X: option '%s' accepted.\n"),
+            logmsg (_("HHCTA066I %4.4X: option '%s' accepted.\n"),
                 dev->devnum, argv[i]);
 
     } // end for (i = 1; i < argc; i++)
@@ -1680,7 +1567,7 @@ void tapedev_query_device ( DEVBLK *dev, char **class,
 
         if ( TAPEDEVT_SCSITAPE != dev->tapedevt
 #if defined(OPTION_SCSI_TAPE)
-            || !STS_NOT_MOUNTED(dev)
+            || STS_MOUNTED(dev)
 #endif
         )
         {
@@ -1734,7 +1621,7 @@ void UpdateDisplay( DEVBLK *dev )
 
         dev->prev_tapemsg = strdup( msgbfr );
 
-        logmsg(_("HHCTA100I %4.4X: Now Displays: %s\n"),
+        logmsg(_("HHCTA010I %4.4X: Now Displays: %s\n"),
             dev->devnum, msgbfr );
     }
 #if defined(OPTION_SCSI_TAPE)
@@ -1754,8 +1641,6 @@ void ReqAutoMount( DEVBLK *dev )
     BYTE   tapeloaded, autoload, mountreq, unmountreq, stdlbled, ascii, scratch;
     char*  lbltype;
     char*  tapemsg = "";
-    char*  eyecatcher =
-"*******************************************************************************";
 
     ///////////////////////////////////////////////////////////////////
 
@@ -1825,9 +1710,18 @@ void ReqAutoMount( DEVBLK *dev )
     if (dev->fd < 0)
     {
         BYTE unitstat = 0, code = 0;
+        BYTE *sensebkup;
+
+        /* Save any pending sense */
+        sensebkup=malloc(dev->numsense);
+        memcpy(sensebkup,dev->sense,dev->numsense);
 
         dev->tmh->open( dev, &unitstat, code );
 
+        /* Restore pending sense */
+        memcpy(dev->sense,sensebkup,dev->numsense);
+        free(sensebkup);
+
 #if defined(OPTION_SCSI_TAPE)
         if (TAPEDEVT_SCSITAPE == dev->tapedevt)
         {
@@ -1931,11 +1825,7 @@ void ReqAutoMount( DEVBLK *dev )
     // Since we currently don't have any way of activating a SCSI tape
     // drive's REAL autoloader mechanism whenever we receive an auto-
     // mount message [from the guest o/s via the Load Display CCW], we
-    // issue a normal operator mount request message instead (in order
-    // to ask the [Hercules] operator (a real human being) to please
-    // perform the automount for us instead since we can't [currently]
-    // do it for them automatically since we don't currently have any
-    // way to send the real request on to the real SCSI device).
+    // leave it to the operator to action the mount message displayed
 
     // Once ASPI code eventually gets added to Herc (and/or something
     // similar for the Linux world), then the following workaround can
@@ -1977,55 +1867,7 @@ void ReqAutoMount( DEVBLK *dev )
                     volser, dev->devnum, dev->filename);
         }
     }
-    else
-    {
-        // If this is a mount or unmount request, inform the
-        // [Hercules] operator of the action they're expected to take...
 
-        if ( unmountreq )
-        {
-            char* keep_or_retain = "";
-
-            if ( 'K' == tapemsg[0] ) keep_or_retain = "and keep ";
-            if ( 'R' == tapemsg[0] ) keep_or_retain = "and retain ";
-
-            if ( scratch )
-            {
-                logmsg(_("\n%s\nAUTOMOUNT: Unmount %sof %s%s scratch tape requested on %4.4X = %s\n%s\n\n"),
-                    eyecatcher,
-                    keep_or_retain,
-                    ascii ? "ASCII " : "",lbltype,
-                    dev->devnum, dev->filename,
-                    eyecatcher );
-            }
-            else
-            {
-                logmsg(_("\n%s\nAUTOMOUNT: Unmount %sof %s%s tape volume \"%s\" requested on %4.4X = %s\n%s\n\n"),
-                    eyecatcher,
-                    keep_or_retain,
-                    ascii ? "ASCII " : "",lbltype,
-                    volser,
-                    dev->devnum, dev->filename,
-                    eyecatcher );
-            }
-        }
-        if ( mountreq )
-        {
-            if ( scratch )
-                logmsg(_("\n%s\nAUTOMOUNT: Mount for %s%s scratch tape requested on %4.4X = %s\n%s\n\n"),
-                    eyecatcher,
-                    ascii ? "ASCII " : "",lbltype,
-                    dev->devnum, dev->filename,
-                    eyecatcher );
-            else
-                logmsg(_("\n%s\nAUTOMOUNT: Mount for %s%s tape volume \"%s\" requested on %4.4X = %s\n%s\n\n"),
-                    eyecatcher,
-                    ascii ? "ASCII " : "",lbltype,
-                    volser,
-                    dev->devnum, dev->filename,
-                    eyecatcher );
-        }
-    }
 
 } /* end function ReqAutoMount */
 
@@ -2214,7 +2056,7 @@ int ldpt=0;
 
 #if defined(OPTION_SCSI_TAPE)
         case TAPEDEVT_SCSITAPE:
-            int_scsi_status_update( dev, 0 );   // (internal call)
+            int_scsi_status_update( dev, 0 );
             if ( STS_BOT( dev ) )
             {
                 dev->eotwarning = 0;
