@@ -1,24 +1,11 @@
-/* TAPEMAP.C   (c) Copyright Jay Maynard, 2000-2009                  */
+/* TAPEMAP.C    (c) Copyright Jay Maynard, 2000-2009                 */
 /*              Map AWSTAPE format tape image                        */
 
-// $Id: tapemap.c 5127 2009-01-23 13:25:01Z bernard $
-
 /*-------------------------------------------------------------------*/
 /* This program reads an AWSTAPE format tape image file and produces */
 /* a map of the tape, printing any standard label records it finds.  */
 /*-------------------------------------------------------------------*/
 
-// $Log$
-// Revision 1.27  2008/11/04 04:50:46  fish
-// Ensure consistent utility startup
-//
-// Revision 1.26  2007/06/23 00:04:18  ivan
-// Update copyright notices to include current year (2007)
-//
-// Revision 1.25  2006/12/08 09:43:31  jj
-// Add CVS message log
-//
-
 #include "hstdinc.h"
 
 #include "hercules.h"
@@ -92,7 +79,7 @@ char            pathname[MAX_PATH];     /* file path in host format  */
 
     /* Open the tape device */
     hostpath(pathname, filename, sizeof(pathname));
-    infd = open (pathname, O_RDONLY | O_BINARY);
+    infd = hopen(pathname, O_RDONLY | O_BINARY);
     if (infd < 0)
     {
         printf ("tapemap: Error opening %s: %s\n",
